<?php

namespace Drupal\asc_courses\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

use \Drupal\asc_courses\AscCoursesApi;
use \Drupal\asc_courses\AscCoursesImporter;

class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'asc_courses_config';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Form constructor
    $form = parent::buildForm($form, $form_state);

    $config = $this->config('asc_courses.settings');

    // D-Org number 
    $form['dept_org'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('D-Org number(s)'),
      '#default_value' => $config->get('asc_courses.dept_org'),
      '#description' => $this->t('D-Org number (i.e. D1435). Multiple numbers may be entered separated by commas'),
    );

    // Import now?
    $form['import_now'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Import now!'),
      // '#default_value' => $config->get('asc_courses.dept_org'),
      '#description' => $this->t('Checking this box will create new and update existing courses.'),
    );
    
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('asc_courses.settings');
    $config->set('asc_courses.dept_org', $form_state->getValue('dept_org'));
    $config->save();

    $import = $form_state->getValue('import_now');
    

    if($import) {
      \Drupal::logger('asc_courses')->notice("Running courses import.");
      $importer = new AscCoursesImporter();
      $importer->fetchAndImportAll();
    }

    
    return parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'asc_courses.settings',
    ];
  }
}

